"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateDialogChild = updateDialogChild;
exports.default = transformDialogTrigger;
const utils_1 = require("../../shared/utils");
const transforms_1 = require("../../shared/transforms");
const getComponents_1 = require("../../../getComponents");
const t = __importStar(require("@babel/types"));
let availableComponents = (0, getComponents_1.getComponents)();
/**
 * Updates DialogTrigger and DialogContainer to the new API.
 *
 * Example:
 * - When `type="popover"`, replaces Dialog with `<Popover>`.
 * - When `type="fullscreen"`, replaces Dialog with `<FullscreenDialog>`.
 * - When `type="fullscreenTakeover"`, replaces Dialog with `<FullscreenDialog variant="fullscreenTakeover">`.
 */
function updateDialogChild(path) {
    let typePath = path.get('openingElement').get('attributes').find((attr) => t.isJSXAttribute(attr.node) && attr.node.name.name === 'type');
    let type = typePath?.node.value?.type === 'StringLiteral' ? typePath.node.value?.value : 'modal';
    let newComponentName = 'Dialog';
    let props = [];
    if (type === 'popover') {
        newComponentName = 'Popover';
    }
    else if (type === 'fullscreen' || type === 'fullscreenTakeover') {
        newComponentName = 'FullscreenDialog';
        if (type === 'fullscreenTakeover') {
            props.push(t.jsxAttribute(t.jsxIdentifier('variant'), t.stringLiteral(type)));
        }
    }
    for (let prop of ['isDismissible', 'mobileType', 'hideArrow', 'placement', 'shouldFlip', 'isKeyboardDismissDisabled', 'containerPadding', 'offset', 'crossOffset']) {
        let attr = path.get('openingElement').get('attributes').find(attr => attr.isJSXAttribute() && attr.node.name.name === prop);
        if (attr) {
            props.push(attr.node);
            attr.remove();
        }
    }
    typePath?.remove();
    let localName = newComponentName;
    if (newComponentName !== 'Dialog' && availableComponents.has(newComponentName)) {
        let program = path.findParent((p) => t.isProgram(p.node));
        localName = (0, utils_1.addComponentImport)(program, newComponentName);
    }
    path.traverse({
        JSXElement(dialog) {
            if (!t.isJSXIdentifier(dialog.node.openingElement.name) || (0, utils_1.getName)(dialog, dialog.node.openingElement.name) !== 'Dialog') {
                return;
            }
            dialog.node.openingElement.name = t.jsxIdentifier(localName);
            if (dialog.node.closingElement) {
                dialog.node.closingElement.name = t.jsxIdentifier(localName);
            }
            dialog.node.openingElement.attributes.push(...props);
        }
    });
}
/**
 * Transforms DialogTrigger:
 * - Comment out type="tray" (it has not been implemented yet).
 * - Comment out mobileType (it has not been implemented yet).
 * - Remove targetRef (it is no longer supported).
 * - Move render props to the child component (updated API).
 */
function transformDialogTrigger(path) {
    // Comment out type="tray"
    (0, transforms_1.commentOutProp)(path, { propName: 'type', propValue: 'tray' });
    // Comment out mobileType
    (0, transforms_1.commentOutProp)(path, { propName: 'mobileType' });
    // Remove targetRef
    (0, transforms_1.removeProp)(path, { propName: 'targetRef' });
    // Move render props to the child component
    (0, transforms_1.moveRenderPropsToChild)(path, { newChildComponentName: 'Dialog' });
    // Update isDismissable to isDismissible
    (0, transforms_1.updatePropName)(path, { oldPropName: 'isDismissable', newPropName: 'isDismissible' });
    // Update DialogTrigger to the new API
    updateDialogChild(path);
}
