"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = transformSearchField;
const transforms_1 = require("../../shared/transforms");
/**
 * Transforms SearchField:
 * - Remove placeholder (it has been removed due to accessibility issues).
 * - Comment out icon (it has not been implemented yet).
 * - Remove isQuiet (it is no longer supported in Spectrum 2).
 * - Change validationState="invalid" to isInvalid.
 * - Remove validationState="valid" (it is no longer supported in Spectrum 2).
 */
function transformSearchField(path) {
    // Remove placeholder
    (0, transforms_1.removeProp)(path, { propName: 'placeholder' });
    // Comment out icon
    (0, transforms_1.commentOutProp)(path, { propName: 'icon' });
    // Remove isQuiet
    (0, transforms_1.removeProp)(path, { propName: 'isQuiet' });
    // Change validationState="invalid" to isInvalid
    (0, transforms_1.updatePropNameAndValue)(path, {
        oldPropName: 'validationState',
        oldPropValue: 'invalid',
        newPropName: 'isInvalid',
        newPropValue: true
    });
    // Remove validationState="valid"
    (0, transforms_1.removeProp)(path, { propName: 'validationState', propValue: 'valid' });
}
