"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = transformTable;
const utils_1 = require("../../shared/utils");
const transforms_1 = require("../../shared/transforms");
const t = __importStar(require("@babel/types"));
/**
 * Copies the columns prop from the TableHeader to the Row component.
 */
function addColumnsPropToRow(path) {
    const tableHeaderPath = path.get('children').find((child) => t.isJSXElement(child.node) &&
        t.isJSXIdentifier(child.node.openingElement.name) &&
        (0, utils_1.getName)(child, child.node.openingElement.name) === 'TableHeader');
    if (!tableHeaderPath) {
        (0, utils_1.addComment)(path.node, ' TODO(S2-upgrade): Could not find TableHeader within Table to retrieve columns prop.');
        return;
    }
    const columnsProp = tableHeaderPath
        .get('openingElement')
        .get('attributes')
        .find((attr) => t.isJSXAttribute(attr.node) && attr.node.name.name === 'columns');
    if (columnsProp) {
        path.traverse({
            JSXElement(innerPath) {
                if (t.isJSXElement(innerPath.node) &&
                    t.isJSXIdentifier(innerPath.node.openingElement.name) &&
                    (0, utils_1.getName)(innerPath, innerPath.node.openingElement.name) === 'Row') {
                    let rowPath = innerPath;
                    rowPath.node.openingElement.attributes.push(columnsProp.node);
                    // If Row doesn't contain id prop, leave a comment for the user to check manually
                    let idProp = rowPath.get('openingElement').get('attributes').find((attr) => t.isJSXAttribute(attr.node) && attr.node.name.name === 'id');
                    if (!idProp) {
                        (0, utils_1.addComment)(rowPath.node, ' TODO(S2-upgrade): If the items do not have id properties, you\'ll need to add an id prop to the Row.');
                    }
                }
            }
        });
    }
}
function commentIfNestedColumns(path) {
    const headerPath = path.get('children').find((child) => t.isJSXElement(child.node) &&
        t.isJSXIdentifier(child.node.openingElement.name) &&
        (0, utils_1.getName)(child, child.node.openingElement.name) === 'TableHeader');
    const columns = headerPath?.get('children') || [];
    let hasNestedColumns = false;
    columns.forEach(column => {
        let columnChildren = column.get('children');
        if (columnChildren.find(child => t.isJSXElement(child.node) &&
            t.isJSXIdentifier(child.node.openingElement.name) &&
            (0, utils_1.getName)(child, child.node.openingElement.name) === 'Column')) {
            hasNestedColumns = true;
        }
    });
    if (hasNestedColumns) {
        (0, utils_1.addComment)(path.node, ' TODO(S2-upgrade): Nested Column components are not supported yet.');
    }
}
/**
 * Adds isRowHeader to the first Column in a table if there isn't already a row header.
 * @param path
 */
function addRowHeader(path) {
    let tableHeaderPath = path.get('children').find((child) => t.isJSXElement(child.node) &&
        t.isJSXIdentifier(child.node.openingElement.name) &&
        (0, utils_1.getName)(child, child.node.openingElement.name) === 'TableHeader');
    // Check if isRowHeader is already set on a Column
    let hasRowHeader = false;
    tableHeaderPath?.get('children').forEach((child) => {
        if (t.isJSXElement(child.node) &&
            t.isJSXIdentifier(child.node.openingElement.name) &&
            (0, utils_1.getName)(child, child.node.openingElement.name) === 'Column') {
            let isRowHeaderProp = child.get('openingElement').get('attributes').find((attr) => t.isJSXAttribute(attr.node) && attr.node.name.name === 'isRowHeader');
            if (isRowHeaderProp) {
                hasRowHeader = true;
            }
        }
    });
    // If there isn't already a row header, add one to the first Column if possible
    if (!hasRowHeader) {
        tableHeaderPath?.get('children').forEach((child) => {
            // Add to first Column if static
            if (!hasRowHeader &&
                t.isJSXElement(child.node) &&
                t.isJSXIdentifier(child.node.openingElement.name) &&
                (0, utils_1.getName)(child, child.node.openingElement.name) === 'Column') {
                child.node.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('isRowHeader'), t.jsxExpressionContainer(t.booleanLiteral(true))));
                hasRowHeader = true;
            }
            // If render function is used, leave a comment to update manually
            if (t.isJSXExpressionContainer(child.node) &&
                t.isArrowFunctionExpression(child.node.expression)) {
                (0, utils_1.addComment)(child.node, ' TODO(S2-upgrade): You\'ll need to add isRowHeader to one of the columns manually.');
            }
            // If array.map is used, leave a comment to update manually
            if (t.isJSXExpressionContainer(child.node) &&
                t.isCallExpression(child.node.expression) &&
                t.isMemberExpression(child.node.expression.callee) &&
                t.isIdentifier(child.node.expression.callee.property) &&
                child.node.expression.callee.property.name === 'map') {
                (0, utils_1.addComment)(child.node, ' TODO(S2-upgrade): You\'ll need to add isRowHeader to one of the columns manually.');
            }
        });
    }
}
/**
 * Transforms TableView:
 * - For Column and Row: Update key to be id (and keep key if rendered inside array.map).
 * - For dynamic tables, pass a columns prop into Row.
 * - For Row: Update dynamic render function to pass in column instead of columnKey.
 * - Comment out UNSTABLE_allowsExpandableRows (it has not been implemented yet).
 * - Comment out UNSTABLE_onExpandedChange (it has not been implemented yet).
 * - Comment out UNSTABLE_expandedKeys (it has not been implemented yet).
 * - Comment out UNSTABLE_defaultExpandedKeys (it has not been implemented yet).
 */
function transformTable(path) {
    // Add columns prop to Row for dynamic tables
    addColumnsPropToRow(path);
    // Comment out nested columns
    commentIfNestedColumns(path);
    // Comment out dragAndDropHooks
    (0, transforms_1.commentOutProp)(path, { propName: 'dragAndDropHooks' });
    // Comment out selectionStyle="highlight"
    (0, transforms_1.commentOutProp)(path, { propName: 'selectionStyle' });
    // Comment out unstable expandable rows props
    (0, transforms_1.commentOutProp)(path, { propName: 'UNSTABLE_allowsExpandableRows' });
    (0, transforms_1.commentOutProp)(path, { propName: 'UNSTABLE_onExpandedChange' });
    (0, transforms_1.commentOutProp)(path, { propName: 'UNSTABLE_expandedKeys' });
    (0, transforms_1.commentOutProp)(path, { propName: 'UNSTABLE_defaultExpandedKeys' });
    addRowHeader(path);
}
