"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = transformTabs;
const utils_1 = require("../../shared/utils");
const transforms_1 = require("../../shared/transforms");
const t = __importStar(require("@babel/types"));
function transformTabList(tabListPath) {
    tabListPath.get('children').forEach(itemPath => {
        if (t.isJSXElement(itemPath.node) &&
            t.isJSXIdentifier(itemPath.node.openingElement.name) &&
            (0, utils_1.getName)(itemPath, itemPath.node.openingElement.name) === 'Item') {
            (0, transforms_1.updateComponentWithinCollection)(itemPath, { parentComponentName: 'TabList', newComponentName: 'Tab' });
        }
    });
    return tabListPath.node;
}
function transformTabPanels(tabPanelsPath, itemsProp) {
    // Dynamic case
    let dynamicRender = tabPanelsPath.get('children').find(path => t.isJSXExpressionContainer(path.node));
    if (dynamicRender) {
        (0, transforms_1.updateToNewComponentName)(tabPanelsPath, { newComponentName: 'Collection' });
        let itemPath = dynamicRender.get('expression').get('body');
        (0, transforms_1.updateComponentWithinCollection)(itemPath, { parentComponentName: 'Collection', newComponentName: 'TabPanel' });
        if (itemsProp) {
            tabPanelsPath.node.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('items'), itemsProp.value));
        }
        return [tabPanelsPath.node];
    }
    // Static case
    return tabPanelsPath.get('children').map(itemPath => {
        if (t.isJSXElement(itemPath.node) &&
            t.isJSXIdentifier(itemPath.node.openingElement.name) &&
            (0, utils_1.getName)(itemPath, itemPath.node.openingElement.name) === 'Item') {
            (0, transforms_1.updateComponentWithinCollection)(itemPath, { parentComponentName: 'TabPanels', newComponentName: 'TabPanel' });
            return itemPath.node;
        }
        return null;
    }).filter(Boolean);
}
/**
 * Transforms Tabs props and structure:
 * - Inside TabList: Update Item to be Tab.
 * - Update items on Tabs to be on TabList.
 * - Inside TabPanels: Update Item to be a TabPanel and remove the surrounding TabPanels.
 * - Remove isEmphasized (it is no longer supported in Spectrum 2).
 * - Remove isQuiet (it is no longer supported in Spectrum 2).
 */
function transformTabs(path) {
    let program = path.findParent((p) => t.isProgram(p.node));
    (0, utils_1.removeComponentImport)(program, 'TabPanels');
    let tabListNode = null;
    let tabPanelsNodes = [];
    let itemsProp = null;
    path.node.openingElement.attributes = path.node.openingElement.attributes.filter(attr => {
        if (t.isJSXAttribute(attr) && attr.name.name === 'items') {
            itemsProp = attr;
            return false;
        }
        return true;
    });
    path.get('children').forEach(childPath => {
        if (t.isJSXElement(childPath.node)) {
            if (t.isJSXIdentifier(childPath.node.openingElement.name) &&
                (0, utils_1.getName)(childPath, childPath.node.openingElement.name) === 'TabList') {
                tabListNode = transformTabList(childPath);
                if (itemsProp) {
                    tabListNode.openingElement.attributes.push(itemsProp);
                }
            }
            else if (t.isJSXIdentifier(childPath.node.openingElement.name) &&
                (0, utils_1.getName)(childPath, childPath.node.openingElement.name) === 'TabPanels') {
                tabPanelsNodes = transformTabPanels(childPath, itemsProp);
            }
        }
    });
    if (tabListNode) {
        path.node.children = [tabListNode, ...tabPanelsNodes];
    }
    // Remove isEmphasized
    (0, transforms_1.removeProp)(path, { propName: 'isEmphasized' });
    // Remove isQuiet
    (0, transforms_1.removeProp)(path, { propName: 'isQuiet' });
}
